@@ -215,7 +215,7 @@ expStateFromChannelName(interp,name,open,adjust,any,msg)
 {
     ExpState *esPtr;
     Tcl_Channel channel;
-    char *chanName;
+    CONST char *chanName;
 
     if (any) {
 	if (0 == strcmp(name,EXP_SPAWN_ID_ANY_LIT)) {
@@ -581,7 +581,7 @@ char **argv;
     char *chanName = 0;
     int leaveopen = FALSE;
     int rc, wc;
-    char *stty_init;
+    CONST char *stty_init;
     int slave_write_ioctls = 1;
 		/* by default, slave will be write-ioctled this many times */
     int slave_opens = 3;
@@ -1312,7 +1312,7 @@ Tcl_Interp *interp;
 struct slow_arg *x;
 {
 	int sc;		/* return from scanf */
-	char *s = exp_get_var(interp,"send_slow");
+	CONST char *s = exp_get_var(interp,"send_slow");
 	if (!s) {
 		exp_error(interp,"send -s: send_slow has no value");
 		return(-1);
@@ -1343,13 +1343,25 @@ struct slow_arg *arg;
 {
 	int rc;
 
+	char *p = buffer;
 	while (rembytes > 0) {
-		int len;
+		int bytelen;
+		int charlen;
+		char *p;
+		int i;
 		
-		len = (arg->size<rembytes?arg->size:rembytes);
-		if (0 > exact_write(esPtr,buffer,len)) return(-1);
-		rembytes -= arg->size;
-		buffer += arg->size;
+		p = buffer;
+		charlen = (arg->size<rembytes?arg->size:rembytes);
+
+		/* count out the right number of UTF8 chars */
+		for (i=0;i<charlen;i++) {
+		  p = Tcl_UtfNext(p);
+		}
+		bytelen = p-buffer;
+
+		if (0 > exact_write(esPtr,buffer,bytelen)) return(-1);
+		rembytes -= bytelen;
+		buffer += bytelen;
 
 		/* skip sleep after last write */
 		if (rembytes > 0) {
@@ -1374,7 +1386,7 @@ Tcl_Interp *interp;
 struct human_arg *x;
 {
 	int sc;		/* return from scanf */
-	char *s = exp_get_var(interp,"send_human");
+	CONST char *s = exp_get_var(interp,"send_human");
 
 	if (!s) {
 		exp_error(interp,"send -h: send_human has no value");
