@@ -156,7 +156,13 @@ int *was_raw, *was_echo;
 
 	if (exp_tty_set_simple(&tty_current) == -1) {
 		expErrorLog("ioctl(raw): %s\r\n",Tcl_PosixError(interp));
-		Tcl_Exit(1);
+
+		/* SF #439042 -- Allow overide of "exit" by user / script
+		 */
+		{
+		  char buffer [] = "exit 1";
+		  Tcl_Eval(interp, buffer); 
+		}
 	}
 
 	exp_ioctled_devtty = TRUE;
@@ -185,7 +191,13 @@ int *was_raw, *was_echo;
 
 	if (exp_tty_set_simple(&tty_current) == -1) {
 		expErrorLog("ioctl(noraw): %s\r\n",Tcl_PosixError(interp));
-		Tcl_Exit(1);
+
+		/* SF #439042 -- Allow overide of "exit" by user / script
+		 */
+		{
+		  char buffer [] = "exit 1";
+		  Tcl_Eval(interp, buffer); 
+		}
 	}
 	exp_ioctled_devtty = TRUE;
 
@@ -201,7 +213,13 @@ int echo;
 {
 	if (exp_tty_set_simple(tty) == -1) {
 		expErrorLog("ioctl(set): %s\r\n",Tcl_PosixError(interp));
-		Tcl_Exit(1);
+
+		/* SF #439042 -- Allow overide of "exit" by user / script
+		 */
+		{
+		  char buffer [] = "exit 1";
+		  Tcl_Eval(interp, buffer); 
+		}
 	}
 	is_raw = raw;
 	is_noecho = !echo;
@@ -656,7 +674,13 @@ char **argv;
 	        if (ioctl(exp_dev_tty, TCGETS, &tty_current) == -1) {
 #endif
 			expErrorLog("ioctl(get): %s\r\n",Tcl_PosixError(interp));
-			Tcl_Exit(1);
+
+			/* SF #439042 -- Allow overide of "exit" by user / script
+			 */
+			{
+			  char buffer [] = "exit 1";
+			  Tcl_Eval(interp, buffer); 
+			}
 		}
 		if (cooked) {
 			/* find out user's new defn of 'cooked' */
@@ -723,7 +747,7 @@ char **argv;
 			(char *) NULL);
 		abnormalExit = TRUE;
 	    } else if (WIFSIGNALED(waitStatus)) {
-		char *p;
+		CONST char *p;
 	
 		p = Tcl_SignalMsg((int) (WTERMSIG(waitStatus)));
 		Tcl_SetErrorCode(interp, "CHILDKILLED", msg1,
@@ -732,7 +756,7 @@ char **argv;
 		Tcl_AppendResult(interp, "child killed: ", p, "\n",
 			(char *) NULL);
 	    } else if (WIFSTOPPED(waitStatus)) {
-		char *p;
+		CONST char *p;
 
 		p = Tcl_SignalMsg((int) (WSTOPSIG(waitStatus)));
 		Tcl_SetErrorCode(interp, "CHILDSUSP", msg1,
