@@ -27,6 +27,17 @@ would appreciate credit if you use this file or parts of it.
 */
 extern char *TclGetRegError();
 
+#if defined(HAVE_PTMX_BSD) && defined(HAVE_PTMX)
+/*
+ * Some systems have both PTMX and PTMX_BSD.
+ * In fact, alphaev56-dec-osf4.0e has /dev/pts, /dev/pty, /dev/ptym,
+ * /dev/ptm, /dev/ptmx, and /dev/ptmx_bsd
+ * Suggestion from Martin Buchholz <martin@xemacs.org> is that BSD
+ * is usually deprecated and so should be here.
+ */
+#undef HAVE_PTMX_BSD
+#endif
+
 /* Linux and Digital systems can be configured to have both.
 According to Ashley Pittman <ashley@ilo.dec.com>, Digital works better
 with openpty which supports 4000 while ptmx supports 60. */
@@ -79,7 +90,7 @@ with openpty which supports 4000 while ptmx supports 60. */
 #  include <sys/strpty.h>
 #endif
 
-#ifdef HAVE_PTMX
+#if defined(HAVE_PTMX) && defined(HAVE_STROPTS_H)
 #  include <sys/stropts.h>
 #endif
 
@@ -490,7 +501,7 @@ exp_getptymaster()
                 break;
             sprintf (slave_name, "%s%s", "/dev/ttyp", num_str);
 
-            master = exp_pty_test (master_name, slave_name, 0, num_str);
+            master = exp_pty_test(master_name,slave_name,'0',num_str);
             if (master >= 0)
                 goto done;
         }
@@ -547,7 +558,7 @@ exp_getptymaster()
 			*slave_bank = *tty_bank;
 			sprintf(tty_num,"%02d",num);
 			strcpy(slave_num,tty_num);
-			master = exp_pty_test(master_name,slave_name,tty_bank,tty_num);
+			master = exp_pty_test(master_name,slave_name,*tty_bank,tty_num);
 			if (master >= 0) goto done;
 		}
 	}
@@ -563,7 +574,7 @@ exp_getptymaster()
 			*slave_bank = *tty_bank;
 			sprintf(tty_num,"%03d",num);
 			strcpy(slave_num,tty_num);
-			master = exp_pty_test(master_name,slave_name,tty_bank,tty_num);
+			master = exp_pty_test(master_name,slave_name,*tty_bank,tty_num);
 			if (master >= 0) goto done;
 		}
 	}
@@ -626,18 +637,18 @@ char *stty_args;
 #if defined(HAVE_PTMX_BSD)
 	if (ioctl (slave, I_LOOK, buf) != 0)
 		if (ioctl (slave, I_PUSH, "ldterm")) {
-			expDiagLogPtrStrStr("ioctl(%s,I_PUSH,\"ldterm\") = %s\n",slave,expErrnoMsg(errno));
+			expDiagLogPtrStrStr("ioctl(%d,I_PUSH,\"ldterm\") = %s\n",slave,expErrnoMsg(errno));
 	}
 #else
 #if defined(HAVE_PTMX)
 	if (ioctl(slave, I_PUSH, "ptem")) {
-		expDiagLogPtrStrStr("ioctl(%s,I_PUSH,\"ptem\") = %s\n",slave,expErrnoMsg(errno));
+		expDiagLogPtrStrStr("ioctl(%d,I_PUSH,\"ptem\") = %s\n",slave,expErrnoMsg(errno));
 	}
 	if (ioctl(slave, I_PUSH, "ldterm")) {
-		expDiagLogPtrStrStr("ioctl(%s,I_PUSH,\"ldterm\") = %s\n",slave,expErrnoMsg(errno));
+		expDiagLogPtrStrStr("ioctl(%d,I_PUSH,\"ldterm\") = %s\n",slave,expErrnoMsg(errno));
 	}
 	if (ioctl(slave, I_PUSH, "ttcompat")) {
-		expDiagLogPtrStrStr("ioctl(%s,I_PUSH,\"ttcompat\") = %s\n",slave,expErrnoMsg(errno));
+		expDiagLogPtrStrStr("ioctl(%d,I_PUSH,\"ttcompat\") = %s\n",slave,expErrnoMsg(errno));
 	}
 #endif
 #endif
